/* Johndan Johnson-Eilola Creative Commons Share-Alike License
* Adapted and mangled from Andy Best's phrases tutorial: 
* http://andybest.net/2009/02/processing-opencv-tutorial-2-bubbles/ 
* and Ira Greenberg's bouncing balls less in _Processing: Creative Coding
* and Computational Art_. Audio sigh by the_justin at Freesound: 
* http://www.freesound.org/samplesViewSingle.php?id=9891. Used under creative commons
* sampling 1.0 license. Music soundtrack removed due to obvious copyright 
* considerations, but you can add one back in: put your own mp3 file
* in the Data folder, then uncomment and modify the relevant code below
* as necessary.
*
* Note necessary libraries (I think minim is included in the default
* Processing install, but you'll need to get openCV from
* http://sourceforge.net/projects/opencvlibrary/
*/

import hypermedia.video.*;          //  Imports the OpenCV library
import ddf.minim.*;

AudioPlayer player;
Minim minim;
AudioSnippet snip; 

OpenCV opencv;                      //  Creates a new OpenCV object
PImage movementImg;                 //  Creates a new pImage to hold the movement image
ArrayList phrases;                  //  Creates an ArrayList to hold the text
int poppedPhrases;
PFont font;                         //  Creates a new font object
int fontsize;                       //  To hold fontsize for phrase size calculations
int l1;                             //  length of a single line in chunks string array
int holdy;                          //  hold phraseY position before bouncing up
String one_line;                    // hold a line of text from chunks ar ray
String chunks[];
int textLoaded;                     // flag to see if text is loaded into array yet 1 = true
int totalLines;                      // number of lines in text file
int startYlines;                    // multiply by i to get starting Y for each line (from to to bottom)

void setup()
{
  size ( 800, 600 );                      //  Window size
  minim = new Minim(this);
  opencv = new OpenCV( this );            //  Initialises the OpenCV library
  opencv.capture( 800, 600 );             //  Sets the capture size to 640 x 480
  movementImg = new PImage( 800, 600 );   //  Initializes the PImage that holds the movement image

  phrases = new ArrayList();

  fontsize = 42;                                   //  Start sort of small
  font = loadFont("GoudyOldStyleT-Bold-48.vlw");        //  Load the font file into memory
  textFont(font, fontsize);     

  textLoaded = 0;                         // initial textloaded (= false)
  
   snip = minim.loadSnippet("9891__the_justin__sigh.mp3");
 // put your own mp3 file in the Data folder and change the code 
 // as necessary in the next line, then remove the // from that line.
//  player = minim.loadFile("song_filename.mp3", 10000);
  player.loop();


}

void draw()
{
  
  // Call Phrases class to instantiate each phrase in the array list, including content, location, and dimensions
  //syntax xloc (random), yloc(random), content, width(calc), height(calc)
  // run only once and then set textLoaded tag to 1 = true
  if (textLoaded != 1) {
    String chunks[]  = loadStrings("bank.txt"); // load string into memory

    totalLines = chunks.length;

    for (int i=0; i < totalLines; i++) {
      //    if (textLoaded != 1)  {
      one_line = chunks[i];
      l1 = one_line.length();
      startYlines = ((height/totalLines) + (fontsize/2));
      print(one_line);
      print(" : l1: ");
      println(l1);
      
      // need to get remainder of i*startYlines and use that to put additional padding between lines somehow
      phrases.add(new Phrase( (width/2-(l1*10/2)), i*startYlines+40,chunks[i], l1*10, fontsize));
      print("x location: ");
      println(width/2-((l1*10)/2));
      // phrases.add(new Phrase(width/2, height/2,chunks[i], (l1*10), fontsize));

    }
    textLoaded = 1;
  }

  opencv.read();                              //  Captures a frame from the camera    
  opencv.flip(OpenCV.FLIP_HORIZONTAL);        //  Flips the image horizontally
  image( opencv.image(), 0, 0 );              //  Draws the camera image to the screen
  opencv.absDiff();                           //  Creates a difference image

  opencv.convert(OpenCV.GRAY);                //  Converts to greyscale
  opencv.blur(OpenCV.BLUR, 3);                //  Blur to remove camera noise
  opencv.threshold(20);                       //  Thresholds to convert to black and white
  movementImg = opencv.image();               //  Puts the OpenCV buffer into an image object

  // check on phrase updates
  for ( int i = 0; i < phrases.size(); i++ ){    //  For every phrase in the phrases array
    Phrase _phrase = (Phrase) phrases.get(i);    //  Copies the current phrase into a temporary object

    if(_phrase.update() == 1){                  //  If the phrases's update function returns '1'

      //     phrases.remove(i);                        //  then remove the phrase from the array
      //     _phrase = null;                           //  and make the temporary phrase object null
      i--;                         //  since we've removed a phrase from the array, we need to subtract 1 from i, or we'll skip th next phrase

    }
    else{                                        //  If the phrase's update function doesn't return '1'
      phrases.set(i, _phrase);                  //  Copies the updated temporary phrase object back into the array
      _phrase = null;                           //  Makes the temporary phrase object null.
    }
  }

  opencv.remember(OpenCV.SOURCE, OpenCV.FLIP_HORIZONTAL);    //  Remembers the camera image so we can generate a difference image next frame. Since we've
  //  flipped the image earlier, we need to flip it here too.
}
class Phrase
{
  int phraseX, phraseY, phraseWidth, phraseHeight;    //  Some variables to hold information about the phrase
  String phraseContent;

  Phrase ( int pX, int pY, String pC, int pW, int pH )           //  The class constructor- sets the values when a new phrase object is made
  {
    phraseX = pX;
    phraseY = pY;
    phraseContent = pC;
    phraseWidth = pW;
    phraseHeight = pH;
  }

  int update()      //   The Phrase update function
  {
    int movementAmount;          //  Create and set a variable to hold the amount of white pixels detected in the area where the phrase is
    int moveleftX, moverightX, leftorright; // figure out where movement happens
 
    moveleftX = 0;
    moverightX = 0;
    leftorright=0;
    movementAmount = 0;


    for( int y = phraseY; y < (phraseY + (phraseHeight-1)); y++ ){    //  For loop that cycles through all of the pixels in the area the phrase occupies
      for( int x = phraseX; x < (phraseX + (phraseWidth-1)); x++ ){
        if ( x < width && x > 0 && y < height && y > 0 ){             //  If the current pixel is within the screen boundaries
          if (brightness(movementImg.pixels[x + (y * width)]) > 127)  //  and if the brightness is above 127 (in this case, if it is white)
          {
            movementAmount++;                                         //  Add 1 to the movementAmount variable.
            leftorright = (x - (phraseX-1));

            if (leftorright > (phraseWidth/2))                       // figuring out if white pixel is on left or right side
            {                                      // if leftorright > half phrase width, touch on right side (so go left)
              moveleftX++;
            }
            else{
              moverightX++;
            }
          }
        }
      }
    }

    if (movementAmount > 50)               //  If more than 50 pixels of movement are detected in the text area
    {  
     phraseY-=5;                           // move up a little bit every time the phrase is touced. 
      if (moverightX > moveleftX) {    // touched on left side
       if (phraseX <= width-phraseWidth) { // if not at right edge, move right
         phraseX+=30;
       }else{
         phraseX-=phraseWidth+80;                      // if at right edge, move left.
       }

      }else{
        if (phraseX < 1) {
          phraseX+=phraseWidth+80;
        }else{
          phraseX-=30;
        }
      }
    }
          
    
/*    if (phraseY >= height-phraseHeight || phraseY <= phraseHeight) {
      ySpeed*=-1;
      phraseY+=ySpeed;
        }
        text(phraseContent, phraseX, phraseY); // draws text on screen in x, y location
        return 0;                           //  Return 0 so that the text object isn't destroyed
    
    }else{                                 //  If less than 5 pixels of movement are detected,


      phraseY += ySpeed; // move the default amount
*/
      if (phraseY >= height-phraseHeight) {
       phraseY -= 25;  // reached bottom; bounce up
      }
     if (phraseY <= 0) { // reached top; move down
        snip.play();
        phraseY+= height-phraseHeight;
        snip.rewind();  
    }
       
       phraseY += random(-2, 2); //add somre random motion to the Y
      
      fill(241, 242, 201);
      text(phraseContent, phraseX, phraseY); // draws text on screen in x, y location
      moveleftX = 0;  // reset leftX and rightX
      moverightX = 0;
      leftorright=0;

      return 0;                              //  Returns 0 so that the text isn't destroyed
    }

  }

void stop()
{
  // always close Minim audio classes when you are done with them
  snip.close();
  minim.stop();
  
  super.stop();
}
